#include <algorithm>
#include <cstdlib>
#include <fstream>
#include <iostream>

#include <rlcsa/misc/utils.h>

#include "sadacount.h"


int
main(int argc, char** argv)
{
  std::cout << "Building Sadakane's document counting structure" << std::endl;
  if(argc < 2)
  {
    std::cout << "Usage: build_sadacount base_name" << std::endl;
    return 1;
  }

  std::string base_name = argv[1];
  std::cout << "Base name: " << base_name << std::endl;
  std::cout << std::endl;

  RLCSA rlcsa(base_name);
  if(!(rlcsa.isOk())) { return 2; }
  rlcsa.printInfo();
  rlcsa.reportSize(true);
  usint size = rlcsa.getSize();

  double start = CSA::readTimer();
  SadaCount sadacount(rlcsa, SadaCount::ENCODING_SMALLEST, true);
  if(!(sadacount.isOk())) { return 3; }
  double total = CSA::readTimer() - start;

  sadacount.writeTo(base_name);

  double megabytes = size / (double)CSA::MEGABYTE;
  double sadacount_size = sadacount.reportSize() / (double)CSA::MEGABYTE;
  std::cout << "Processed " << megabytes << " megabytes in " << total << " seconds (" << (megabytes / total) << " MB/s)." << std::endl;
  std::cout << "Memory: " << CSA::memoryUsage() << " kB" << std::endl;
  std::cout << "Size: " << sadacount_size << " MB (" << (CHAR_BIT * sadacount_size / megabytes) << " bpc)" << std::endl;
  std::cout << std::endl;

  return 0;
}
