#include <algorithm>
#include <fstream>
#include <iostream>
#include <stack>

#include "pdlcount.h"

//--------------------------------------------------------------------------

PDLCount::PDLCount(const RLCSA& _rlcsa, const std::string& base_name, bool _fast) :
  rlcsa(_rlcsa), status(st_error), fast(_fast),
  tree(0), counts(0)
{
  if(!(this->rlcsa.isOk()) || !(this->rlcsa.supportsLocate()))
  {
    std::cerr << "PDLCount::PDLCount(): Invalid RLCSA!" << std::endl;
    return;
  }

  std::string input_name = base_name + (this->fast ? PDLTree::FAST_COUNT_EXTENSION : PDLTree::COUNT_EXTENSION);
  std::ifstream input(input_name.c_str(), std::ios_base::binary);
  if(!input)
  {
    std::cerr << "PDLCount::PDLCount(): Cannot open input file " << input_name << "!" << std::endl;
    return;
  }
  this->tree = new PDLTree(this->rlcsa, input);
  if(!(this->tree->isOk())) { return; }
  this->counts = new CSA::Array(input);
  input.close();

  this->status = st_ok;
}

PDLCount::~PDLCount()
{
  delete this->tree; this->tree = 0;
  delete this->counts; this->counts = 0;
}

void
PDLCount::writeTo(const std::string& base_name) const
{
  if(!(this->isOk()))
  {
    std::cerr << "PDLCount::writeTo(): Status is not ok!" << std::endl;
    return;
  }

  std::string filename = base_name + PDLTree::COUNT_EXTENSION;
  std::ofstream output(filename.c_str(), std::ios_base::binary);
  if(!output)
  {
    std::cerr << "PDLCount::writeTo(): Cannot open output file " << filename << "!" << std::endl;
    return;
  }

  this->tree->writeTo(output);
  this->counts->writeTo(output);
  output.close();
}

usint
PDLCount::reportSize() const
{
  usint bytes = sizeof(*this) + this->tree->reportSize() + this->counts->reportSize();
  return bytes;
}

//--------------------------------------------------------------------------

usint
PDLCount::count(const std::string& pattern) const
{
  if(!(this->isOk())) { return 0; }
  pair_type sa_range = this->rlcsa.count(pattern);
  return this->count(sa_range);
}

usint
PDLCount::count(pair_type sa_range) const
{
  if(!(this->isOk()) || CSA::isEmpty(sa_range) || sa_range.second >= this->rlcsa.getSize()) { return 0; }

  usint block = this->tree->getBlock(sa_range);
  if(block >= this->tree->getNumberOfNodes())
  {
    if(this->fast) { return CSA::length(sa_range); }
    else { return bruteForceDocCount(this->rlcsa, sa_range); }
  }

  CSA::Array::Iterator iter(*(this->counts));
  return iter.readItem(block);
}

//--------------------------------------------------------------------------
