#ifndef _PDLTOPK_DOCLIST_H
#define _PDLTOPK_DOCLIST_H

#include <stack>
#include <vector>

#include <rlcsa/rlcsa.h>

#include "pdltree.h"
#include "utils.h"


using CSA::RLCSA;
using CSA::usint;
using CSA::pair_type;


struct IntermediateDoc
{
  uint id, freq;
  uint blocks, estimate;

  IntermediateDoc(uint _id, uint _freq, uint _blocks) :
    id(_id), freq(_freq), blocks(_blocks), estimate(0)
  {
  }

  inline void updateEstimate(std::vector<uint>& cumulative_freqs)
  {
    this->estimate = this->freq + cumulative_freqs[this->blocks];
  }

  inline Document getDoc() { return Document(this->id, this->freq); }
};

class PDLTopk
{
  public:
    typedef std::vector<Document>                      result_type;
    typedef std::vector<IntermediateDoc>               intermediate_type;
    typedef enum { st_unfinished, st_error, st_ok }    status_type;
    typedef enum { merge_brute, merge_lin, merge_exp } merge_type;

    const static std::string EXTENSION; // .topk
    const static std::string MERGE_EXTENSION; // .topk_merge

    // If compress_sets is true, sets and grammar will be read from the files compressed by irepair.
    PDLTopk(const RLCSA& _rlcsa, const std::string& base_name, bool compress_sets, bool merge);
    ~PDLTopk();

    inline bool isOk() const { return (this->status == st_ok); }
    inline bool supportsMerge() const { return this->supports_merge; }

    void writeTo(const std::string& base_name) const;
    usint reportSize(bool include_freqs) const;

    // User must delete the result.
    result_type* query(const std::string& pattern, usint k, bool get_freqs) const;
    result_type* query(pair_type sa_range, usint k, bool get_freqs) const;
    result_type* query(const std::string& pattern, usint k, merge_type strategy) const;
    result_type* query(pair_type sa_range, usint k, merge_type strategy) const;

  private:
    const RLCSA&     rlcsa;
    status_type      status;
    bool             supports_merge;

    PDLTree*         tree;
    CSA::MultiArray* freqs;
    CSA::ReadBuffer* grammar;
    CSA::MultiArray* blocks;

    result_type* queryUnsafe(pair_type sa_range, usint k, bool get_freqs) const;
    result_type* queryUnsafe(pair_type sa_range, usint k, merge_type strategy) const;

    // k == 0 means all documents.
    void addBlock(result_type* results, usint block, usint k, bool get_freqs) const;

    // Returns true if all blocks are finished.
    bool addFromBlocks(intermediate_type& doc_buffer, usint blocks, usint amount,
      CSA::MultiArray::Iterator** block_iters, CSA::MultiArray::Iterator** freq_iters,
      std::vector<pair_type>& next_freqs, std::stack<usint>* buffers) const;

    // These are not allowed.
    PDLTopk();
    PDLTopk(const PDLTopk&);
    PDLTopk& operator = (const PDLTopk&);
};


#endif  // _PDLTOPK_DOCLIST_H
