#include <algorithm>
#include <cstdlib>
#include <fstream>
#include <iostream>
#include <vector>

#include <rlcsa/misc/utils.h>

#include "sadacount.h"


#ifdef MULTITHREAD_SUPPORT
#include <omp.h>
#endif


const usint ALPHABET_SIZE = 64;
const usint DOCUMENT_LENGTH = 1024;

const usint MIN_DOCS = 1024;
const usint DOCS_MULTIPLIER = 4;
const usint MAX_DOCS = 65536;

const usint RATE_SCALE = 1000;
const usint RATE_MULTIPLIER = 10;


struct TestResult
{
  double rate;
  usint docs;
  usint runs;
};


int
main(int argc, char** argv)
{
  std::cout << "Sadakane's document counting structure analyzer" << std::endl;
/*  if(argc < 2)
  {
    std::cout << "Usage: runs_in_sadacount base_name" << std::endl;
    return 1;
  }

  std::string base_name = argv[1];
  std::cout << "Base name: " << base_name << std::endl;*/
  std::cout << std::endl;

  std::cout << "Rate";
  for(usint docs = MIN_DOCS; docs <= MAX_DOCS; docs *= DOCS_MULTIPLIER)
  {
    std::cout << ";" << docs;
  }
  std::cout << std::endl;

  srand(0xDEADBEEF);  // FIXME random seed
  unsigned char* document = new unsigned char[DOCUMENT_LENGTH];
  for(usint i = 0; i < DOCUMENT_LENGTH; i++) { document[i] = rand() % ALPHABET_SIZE + 1; }

  for(usint rate = 1; rate <= RATE_SCALE; rate *= RATE_MULTIPLIER)
  {
    TestResult result;
    result.rate = rate / (double)RATE_SCALE;
    std::cout << result.rate; std::cout.flush();
    for(result.docs = MIN_DOCS; result.docs <= MAX_DOCS; result.docs *= DOCS_MULTIPLIER)
    {
      usint total_size = result.docs * (DOCUMENT_LENGTH + 1);
      unsigned char* data = new unsigned char[total_size];
      for(usint doc = 0, j = 0; doc < result.docs; doc++)
      {
        for(usint i = 0; i < DOCUMENT_LENGTH; i++, j++)
        {
          if(rand() / (RAND_MAX + 1.0) < result.rate) { data[j] = rand() % ALPHABET_SIZE + 1; }
          else { data[j] = document[i]; }
        }
        data[j] = 0; j++;
      }

      usint threads = 1;
      #ifdef MULTITHREAD_SUPPORT
      threads = omp_get_max_threads();
      #endif
      RLCSA rlcsa(data, total_size, 32, 32, threads, true);
      if(!(rlcsa.isOk())) { return 2; }
      SadaCount sadacount(rlcsa, SadaCount::ENCODING_RLE, false);
      if(!(sadacount.isOk())) { return 3; }
      result.runs = sadacount.countRuns();
      std::cout << ";" << result.runs; std::cout.flush();
    }
    std::cout << std::endl;
  }
  delete[] document; document = 0;
  std::cout << std::endl;

  return 0;
}
