#ifndef _DOCLIST_SADACOUNT_H
#define _DOCLIST_SADACOUNT_H

#include <rlcsa/rlcsa.h>

#include "utils.h"


using CSA::RLCSA;
using CSA::usint;
using CSA::pair_type;


class SadaCount
{
  public:
    const static std::string EXTENSION;

    const static usint ENCODING_MASK     = 0xF;
    const static usint ENCODING_SMALLEST = 0x0;
    const static usint ENCODING_RLE      = 0x1;
    const static usint ENCODING_DELTA    = 0x2;
    const static usint ENCODING_SUCCINCT = 0x3;

    const static usint BLOCK_SIZE        = 32;

    typedef enum { st_unfinished, st_error, st_ok } status_type;

    explicit SadaCount(const RLCSA& _rlcsa, usint _encoding = ENCODING_SMALLEST, bool print = false);
    SadaCount(const RLCSA& _rlcsa, const std::string& base_name);
    ~SadaCount();

    inline bool isOk() const { return (this->status == st_ok); }
    inline usint getEncoding() const { return this->encoding; }

    void writeTo(const std::string& base_name) const;
    usint reportSize() const;
    usint countRuns() const;

    usint count(const std::string& pattern) const;
    usint count(pair_type sa_range) const;

  private:
    const RLCSA&     rlcsa;
    status_type      status;
    usint            encoding;

    // Unlike in the original paper, we encode h(v) as an 1-bit, followed by h(v) 0-bits.
    CSA::RLEVector*      data_rle;
    CSA::DeltaVector*    data_delta;
    CSA::SuccinctVector* data_succinct;

    // These are not allowed.
    SadaCount();
    SadaCount(const SadaCount&);
    SadaCount& operator = (const SadaCount&);
};


#endif  // _DOCLIST_SADACOUNT_H
