#ifndef _DOCLIST_UTILS_H
#define _DOCLIST_UTILS_H

#include <fstream>
#include <vector>

#include <rlcsa/rlcsa.h>


using CSA::usint;
using CSA::pair_type;
using CSA::RLCSA;

// This is also defined in namespace CSA.
template<class A, class B>
std::ostream& operator<<(std::ostream& stream, const std::pair<A, B>& data)
{
  return stream << "(" << data.first << ", " << data.second << ")";
}

// User must delete the returned buffer.
CSA::ReadBuffer* readDocarray(const RLCSA& rlcsa, const std::string& base_name);
CSA::ReadBuffer* readDocarray(const RLCSA& rlcsa, std::ifstream& input);


void printResults(const std::string& algorithm, double seconds, usint docc, usint patterns);
void printSize(const std::string& algorithm, usint text_size, usint index_size, usint doclist_size);
void printSizeNoCSA(const std::string& algorithm, usint text_size, usint doclist_size);


typedef std::pair<uint, uint> Document; // (id, freq)

void sortById(std::vector<Document>* docs, bool parallel);

// Descending by freq, then ascending by id.
void sortByFrequency(std::vector<Document>* docs, bool parallel);

// Merge runs of identical documents.
void mergeDocumentRuns(std::vector<Document>* docs);


// User must delete the returned vector.
// If a result vector is given, the return value is always the same vector. In that case, the user
// is responsible for filtering out duplicates.
std::vector<uint>* bruteForceDocList(const RLCSA& rlcsa, const std::string& pattern, std::vector<uint>* result = 0);
std::vector<uint>* bruteForceDocList(const RLCSA& rlcsa, pair_type sa_range, std::vector<uint>* result = 0);
std::vector<uint>* bruteForceDocList(const RLCSA& rlcsa, const CSA::ReadBuffer& docarray, const std::string& pattern, std::vector<uint>* result = 0);
std::vector<uint>* bruteForceDocList(const CSA::ReadBuffer& docarray, pair_type sa_range, std::vector<uint>* result = 0);

usint bruteForceDocCount(const RLCSA& rlcsa, const std::string& pattern);
usint bruteForceDocCount(const RLCSA& rlcsa, pair_type sa_range);
usint bruteForceDocCount(const RLCSA& rlcsa, const CSA::ReadBuffer& docarray, const std::string& pattern);
usint bruteForceDocCount(const CSA::ReadBuffer& docarray, pair_type sa_range);

// User must delete the returned vector.
std::vector<Document>* bruteForceTopk(const RLCSA& rlcsa, const std::string& pattern, usint k);
std::vector<Document>* bruteForceTopk(const RLCSA& rlcsa, pair_type sa_range, usint k);
std::vector<Document>* bruteForceTopk(const RLCSA& rlcsa, const CSA::ReadBuffer& docarray, const std::string& pattern, usint k);
std::vector<Document>* bruteForceTopk(const CSA::ReadBuffer& docarray, pair_type sa_range, usint k);


#endif  // _DOCLIST_UTILS_H
